﻿define("fotowareintegration/editors/FotoWareThumbnailSelector", [
    // dojo
    "dojo/_base/declare",
    "dojo/when",
    "dojo/on",
    "dojo/topic",
    "dojo/dom-class",
    "dojo/ready",
    "dojo/_base/lang",
    // dijit
    "dijit/focus",
    "dijit/ProgressBar",
    // epi
    "epi/Url",
    "epi/routes",
    "epi/shell/dnd/Target",
    "epi/shell/DialogService",
    "epi/shell/XhrWrapper",
    // epi-cms
    "epi-cms/core/ContentReference",
    "epi-cms/widget/MediaSelector",
    "epi-cms/widget/ThumbnailSelector",
    "epi-cms/command/UploadContent",
    "epi-cms/widget/UploadUtil",
    "epi-cms/widget/viewmodel/MultipleFileUploadViewModel",
    "epi-cms/widget/LocalFolderUploader",
    "dojo/text!./templates/FotoWareThumbnailSelector.html",
    "dojo/text!./templates/FotoWareThumbnailSelectorDisabledDefault.html",
    //"fotowareintegration/ed/LegacyDialogPopup",
    "dojo/text!./templates/dialog.html",
    "fotowareintegration/editors/dialog",
    "epi/i18n!epi/cms/nls/episerver.cms.widget.thumbnailselector"
], function (
    // dojo
    declare,
    when,
    on,
    topic,
    domClass,
    ready,
    lang,
    // dijit
    focusManager,
    ProgressBar,
    // epi
    Url,
    routes,
    Target,
    dialogService,
    XhrWrapper,
    // epi-cms
    ContentReference,
    MediaSelector,
    ThumbnailSelector,
    UploadContent,
    UploadUtil,
    MultipleFileUploadViewModel,
    LocalFolderUploader,
    template,
    templateEpiDisabled,
    templateDialog,
    Dialog,
    resources
) {
    var xhr = new XhrWrapper();
    return declare([ThumbnailSelector], {
        // summary:
        //      Represents the widget to select ContentReference.
        // tags:
        //      internal

        resources: resources,
        loaded: false,
        uploadDisabledMessage: "Drag and drop feature has been disabled, please use Fotoware.",
        pluginpath: null,
        pluginversion: null,
        pluginiconpath: null,
        plugintenanturl: null,
        pluginmodulepath: null,
        pluginWindow: null,
        activeElement: null,
        fwWin: null,
        pluginurl: null,
        timer: null,
        disableMediaSelector: null,
        listenerBound: false,
        //      List of file extensions that may be dropped onto the editor
        allowedExtensions: [],
        filesNotOpenInExportApi: null,
        templateString: template,
        buildRendering: function () {
            //switching template depending on functionality
            this.templateString = this.disableMediaSelector === "True" ? templateEpiDisabled : template;
            this.inherited(arguments);
        },
        postCreate: function () {

            ready(setTimeout(lang.hitch(this, function () {//not optimal with timeout, but else not working with _setValue when used on init
                this.loaded = true;
            }), 100));

            this.allowedExtensions = this.pluginallowedfiletypes.replace(".", "").replace(/,/gi, "").split('.');

            this.inherited(arguments);
        },
        getFetchCf: function (url, data) {
            var self = this;
            return xhr.xhrGet({
                url: url,
                content: data,
                failOk: true,
                error: function (response) {
                    if (response.xhr.statusText !== "")
                        self._dialogService.alert(response.xhr.statusText);
                    else {
                        var mess = response.response.text.substring(0, response.response.text.length > 1500 ? 1500 : response.response.text.length);
                        self._dialogService.alert(mess);
                    }
                }
            });
        },
        getCookie: function (name) {
            var value = "; " + document.cookie;
            var parts = value.split("; " + name + "=");
            if (parts.length == 2) return parts.pop().split(";").shift();
            return "";
        },
        popupWinCenter: function (url, title, w, h) {
            var left = (screen.width / 2) - (w / 2);
            var top = (screen.height / 2) - (h / 2);

            var newWindow = window.open(url, title, 'resizable=yes, scrollbars, width=' + w + ', height=' + h + ', top=' + top + ', left=' + left);

            // Puts focus on the newWindow
            if (newWindow.focus) newWindow.focus();

            return newWindow;
        },
        // Custom function for Fotoware integration to trigger opening of dialog.
        _onFotoWareButtonClick: function () {

            this.pluginurl = this.pluginpath + "ClientResources/plugin/plugin.html?pluginBaseUrl=" +
                this.plugintenanturl +
                "/fotoweb/widgets/selection&insertMultiple=false&cn=" + this.plugincookiename + "&";

            var self = this;

            //if cookie not set, must oauth in new window, not in iframe
            if (this.getCookie(this.plugincookiename) === "") {

                require(["dijit/focus"], function (focusUtil) {// focus is lost, so store it temporarly
                    self.activeElement = focusUtil.curNode; // returns null if there is no focused element
                });

                this.fwWin = this.popupWinCenter(this.pluginurl + Math.random(), this.name, 1200, 800);

                if (!this.listenerBound) {
                    this._listener = this._listener.bind(this);
                    window.pluginWindow = this.fwWin;
                    window.addEventListener("message", this._listener, true);
                    this.listenerBound = true;
                }

                return;
            }

            if (self.activeElement !== null) {//set focus if saved and needed
                require(["dijit/focus"], function (focusUtil) {
                    focusUtil.curNode = self.activeElement;
                    self.activeElement = null;
                });
            }

            //this.fwWin = new LegacyDialogPopup({
            //    url: this.pluginurl + Math.random(),
            //    onCallback: null,
            //    dialogArguments: {},
            //    dialogTitle: (this.pluginenableexport ? "Fotoware Export Widget v" : "Fotoware Selection Widget v") + this.pluginversion,
            //    autoFit: true,
            //    showCancelButton: true,
            //    showLoadingOverlay: true,
            //    features: { width: 1200, height: 900 }
            //});
            //this.fwWin.actionContainerNode.hidden = true;

            this.fwWin = new Dialog({
                title: (this.pluginenableexport ? "Fotoware Export Widget v" : "Fotoware Selection Widget v") + this.pluginversion,
                style: "width: 90%; height: 90%;",
                templateString: templateDialog,
                pluginUrl: this.pluginurl + Math.random(),
                pluginversion: this.pluginversion,
                //lang: dependency.resolve("epi.cms.contentEditing.command.Editing").model.languageContext.language,
                //contentid: dependency.resolve("epi.cms.contentEditing.command.Editing").model.contentLink,
                propname: this.get("name")
            });

            this.fwWin.show();

            if (!this.listenerBound) {
                this._listener = this._listener.bind(this);
                window.pluginWindow = this.fwWin;
                window.addEventListener("message", this._listener, true);
                this.listenerBound = true;
            }

            this.timer = setInterval(lang.hitch(this,
                function () {
                    if (!this.fwWin.open) {
                        window.removeEventListener("message", this._listener, true);
                        this.listenerBound = false;
                        clearInterval(this.timer);
                        this.fwWin.destroy();
                        return;
                    }
                }),
                2000);

        },
        _listener: function (event) {

            if (event.data == "loggedinSuccess") {
                this.fwWin.close();
                this._onFotoWareButtonClick();
                return;
            }
            else if (event.data.event === 'selectionWidgetCancel') {
                this.fwWin.hide();
                //remove listener
                window.removeEventListener("message", this._listener, true);
                this.listenerBound = false;
            }
            else if (event.data.event === 'assetSelected') {
                if (this.pluginenableexport && !this.checkFileExtension(event.data.asset.href)) {
                    this.fwWin.containerNode.firstElementChild.src = this.pluginpath +
                        "ClientResources/plugin/plugin.html?pluginBaseUrl=" +
                        this.plugintenanturl +
                        "/fotoweb/widgets/publish&i=" +
                        encodeURIComponent(event.data.asset.href) +
                        "&" + Math.random() +
                        "&cn=" + this.plugincookiename +
                        "&pass=true&" + this.pluginextraquerystring;
                    return;
                } else {
                    //get the default image
                    this.fwWin.hide();

                    this.assetChoosen(event.data.asset.href);

                    //remove listener
                    window.removeEventListener("message", this._listener, true);
                    this.listenerBound = false;
                }

            }
            else if (event.data.event === 'assetExported') {
                if (this.fwWin.hide) //depending on dialog or window.open
                    this.fwWin.hide();

                this.assetChoosenExport(event.data.export.export.source.replace("http://", "https://").replace(this.plugintenanturl, ""), event.data.export.export.image.normal.replace("http://", "https://").replace(this.plugintenanturl, ""));

                //remove listener
                window.removeEventListener("message", this._listener, true);
                this.listenerBound = false;

            }
            //there are more events sent, like frame load, page load, focus


        },
        assetChoosen: function (assethref) {
            var self = this;
            when(self.getFetchCf(
                self.pluginmodulepath + "authentication/fetchcf",
                {
                    hostcontent: "",
                    image: assethref
                }),
                function (data) {
                    self.focus();
                    window.focus();
                    self._setValueAndFireOnChange(data + "");
                });
        },
        assetChoosenExport: function (assethref, exporturl) {
            var self = this;
            when(self.getFetchCf(
                self.pluginmodulepath + "authentication/fetchcf",
                {
                    hostcontent: "",
                    image: assethref,
                    exporturl: exporturl
                }),
                function (data) {
                    self.focus();
                    window.focus();
                    self._setValueAndFireOnChange(data + "");
                });
        },
        //if you want to get other thumbnail from fotoware for types that episerver do not have thumbnails
        //_updateDisplayNode: function (content) {
        //    this.inherited(arguments);
        //    if (content) {
        //        this.thumbnail.src = this._getThumbnailUrl(content).toString();
        //        //custom thumbnail
        //        if (!content.name.match(/.(jpg|jpeg|png|gif|bmp|jpe)$/i)) {
        //            this.thumbnail.src = this.pluginmodulepath + "authentication/GenerateThumb?contentid=" + content.contentLink;
        //            this.thumbnail.style = "object-fit:contain;";
        //        }
        //    }
        //    this.stateNode.title = content ? content.name : "";
        //},
        //buildUrl: function (url, assethref) {

        //    var baseUrl = url + '?i=' + assethref;

        //    baseUrl = baseUrl + this.getUrlPart('&w=', this.defaultWidth);
        //    baseUrl = baseUrl + this.getUrlPart('&h=', this.defaultHeight);
        //    baseUrl = baseUrl + this.getUrlPart('&p=', this.defaultPreset);

        //    return baseUrl;
        //},
        getUrlPart: function (key, value) {
            if (typeof (value) != 'undefined' && value != "") {
                return key + value;
            }
            return "";
        },
        checkFileExtension: function (fileName) {
            let filesNotOpenInExportApi = this.filesNotOpenInExportApi || ".png,.svg,.gif,.pdf";
            let extensions = filesNotOpenInExportApi.toLowerCase().split(',').map(ext => ext.trim() + ".info");
            for (let ext of extensions) {
                if (fileName.toLowerCase().endsWith(ext)) {
                    return true;
                }
            }
            return false;
        },

        _setValueAttr: function (value) {
            if (this.disableMediaSelector === "True" && this.loaded && value != null) {
                this._dialogService.alert(this.uploadDisabledMessage);
                return;
            }
            this.inherited(arguments);
        },
        _onClick: function () {
            if (this.readOnly) {
                return;
            }

            if (this.disableMediaSelector === "True") {
                this._onFotoWareButtonClick();
                return;
            }


            this.inherited(arguments);
        },

        _onDrop: function () {
            if (this.readOnly) {
                event.preventDefault();
                event.stopPropagation();
                return;
            }

            if (this.disableMediaSelector === "True") {
                this._dialogService.alert(this.uploadDisabledMessage);
                event.preventDefault();
                event.stopPropagation();
                return;
            }

            this.inherited(arguments);
        },
        _onButtonClick: function () {
            if (this.readOnly) {
                event.preventDefault();
                event.stopPropagation();
                return;
            }

            if (this.disableMediaSelector === "True") {
                this._onFotoWareButtonClick();
                return;
            }


            this.inherited(arguments);
        },
        _selectContent: function () {
            if (this.readOnly) {
                event.preventDefault();
                event.stopPropagation();
                return;
            }

            if (this.disableMediaSelector === "True") {
                this._onFotoWareButtonClick();
                return;
            }


            this.inherited(arguments);
        }
    });
});
