﻿using EPiServer.Core;
using EPiServer.ServiceLocation;
using Fotoware.Plugins.Optimizely.Interfaces;
using Microsoft.Extensions.Configuration;
using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.IO;
using System.Linq;

namespace Fotoware.Plugins.Optimizely
{
    public class ExampleSiteOneFotoWareSettings : IFotoWareSettings
    {
        private IConfiguration _configuration = ServiceLocator.Current.GetInstance<IConfiguration>();

        public ExampleSiteOneFotoWareSettings()
        {
            MetadataConfiguration = new NameValueCollection
            {
                {"5", "Name"},
                {"25", "Keywords"},
                {"80", "Photographers"},
                {"116", "Copyright"},
                {"120", "Description"}
            };
        }

        public virtual string AllowedTypes => _configuration["FotoWare:AllowedTypes"]?? ".jpg,.jpeg,.jpe,.gif,.bmp,.png";

        public virtual string ClientId => _configuration["FotoWare:ClientId"];

        public virtual string ClientSecret => _configuration["FotoWare:ClientSecret"];

        public virtual string ServerClientId => _configuration["FotoWare:ServerApi:ClientId"];

        public virtual string ServerClientSecret => _configuration["FotoWare:ServerApi:ClientSecret"];

        public string ServerAccessToken { get; set; }

        public DateTime ServerAccessTokenExpireTime { get; set; }

        public virtual string FotoWareTenantUrl => _configuration["FotoWare:TenantUrl"];

        /// <summary>
        /// Used in sheduled Job for full import, not MultiSiteSupport
        /// </summary>       
        public virtual IEnumerable<string> ArchiveUrlsToSynchronize
        {
            get
            {
               var archives = _configuration["FotoWare:ArchiveUrlsToSynchronize"];
               if (!string.IsNullOrWhiteSpace(archives))
               {
                   return archives.Split(',');
               }

               return Enumerable.Empty<string>();
            }
        }


        public virtual NameValueCollection MetadataConfiguration { get; set; }

        public virtual int PreferredDownloadWidth
        {
            get
            {
                if (!int.TryParse(_configuration["FotoWare:PreferredDownloadWidth"], out int size))
                {
                    size = 0;
                }

                return size;
            }
        }

        public virtual string PresetUrl => _configuration["FotoWare:PresetUrl"];

        public virtual string RedirectUri => _configuration["FotoWare:RedirectUri"];

        public virtual bool UseOauth => ServerClientId != null && ServerClientSecret != null;


        public virtual ContentReference RootFolderId

        {
            get
            {
                if (!int.TryParse(_configuration["FotoWare:RootFolderId"], out int id))
                {
                    throw new Exception("FotoWare configuration not complete, FotoWare:RootFolderId in appsettings web.config not set to a valid folder id");
                }
                return new ContentReference(_configuration["FotoWare:RootFolderId"]);
            }
        }

        public virtual bool DisableEpiserverDefaultMediaSelector
        {
            get
            {
                if (!bool.TryParse(_configuration["FotoWare:DisableEpiserverDefaultMediaSelector"], out bool value))
                {
                    value = false;
                }

                return value;
            }
        }

        public virtual bool EnableOnAllImages
        {
            get
            {
                if (!bool.TryParse(_configuration["FotoWare:EnableOnAllImages"], out bool value))
                {
                    value = true;
                }

                return value;
            }
        }

        public virtual bool ExportWidgetEnableOnAllImages
        {
            get
            {
                if (!bool.TryParse(_configuration["FotoWare:ExportWidgetEnableOnAllImages"], out bool value))
                {
                    value = true;
                }

                return value;
            }
        }
    }
}